@@ -2,6 +2,7 @@ use std::{
     fmt::Display,
     fs::{self, create_dir_all, metadata},
     path::{Path, PathBuf},
+    process::exit,
 };
 
 use indicatif::{MultiProgress, ProgressBar, ProgressFinish, ProgressStyle};
@@ -12,31 +13,37 @@ const RETINA_SCALE: u32 = 2;
 pub enum ProgressBarType {
     Input,
     Conversion,
-    OutputWithIcns,
-    OutputWithoutIcns,
+    OutputWithAssignment,
+    OutputIcns,
 }
 
 impl ProgressBarType {
-    pub fn num_steps(&self) -> u64 {
+    pub fn num_steps(&self, options: &Options) -> u64 {
         match self {
             ProgressBarType::Input => 1,
-            ProgressBarType::Conversion => 13,
-            ProgressBarType::OutputWithIcns => 7,
-            ProgressBarType::OutputWithoutIcns => 1,
+            ProgressBarType::Conversion => 13 + if options.badge.is_some() { 1 } else { 0 },
+            ProgressBarType::OutputWithAssignment => {
+                2 + if matches!(options.set_icon_using, SetIconUsing::Rez) {
+                    7
+                } else {
+                    0
+                }
+            }
+            ProgressBarType::OutputIcns => 1,
         }
     }
 }
 
 use crate::{
     command::{
-        run_command, run_convert, DEREZ_COMMAND, ICONUTIL_COMMAND, REZ_COMMAND, SETFILE_COMMAND,
-        SIPS_COMMAND,
+        run_command, run_magick, DEREZ_COMMAND, FILEICON_COMMAND, ICONUTIL_COMMAND,
+        OSASCRIPT_COMMAND, REZ_COMMAND, SETFILE_COMMAND, SIPS_COMMAND,
     },
-    convert::{density, BlurDown, CommandArgs, CompositingOperation},
     error::{FolderifyError, GeneralError},
-    generic_folder_icon::get_folder_icon,
-    options::{self, ColorScheme, Options},
+    magick::{density, BlurDown, CommandArgs, CompositingOperation},
+    options::{Badge, ColorScheme, FolderStyle, Options, SetIconUsing},
     primitives::{Dimensions, Extent, Offset, RGBColor},
+    resources::{get_badge_icon, get_folder_icon, IconInputs},
 };
 
 pub struct ScaledMaskInputs {
@@ -75,10 +82,11 @@ impl WorkingDir {
         progress_bar_type: ProgressBarType,
         stage_description: &str,
         multi_progress_bar: Option<MultiProgress>,
+        options: &Options,
     ) -> IconConversion {
         let progress_bar = match multi_progress_bar {
             Some(multi_progress_bar) => {
-                let progress_bar = ProgressBar::new(progress_bar_type.num_steps());
+                let progress_bar = ProgressBar::new(progress_bar_type.num_steps(options));
                 let progress_bar = match progress_bar_type {
                     ProgressBarType::Conversion => multi_progress_bar.insert(1, progress_bar),
                     _ => multi_progress_bar.insert_from_back(0, progress_bar),
@@ -256,11 +264,6 @@ pub struct IconConversion {
     pub progress_bar: Option<ProgressBar>,
 }
 
-pub struct IconInputs {
-    pub color_scheme: ColorScheme,
-    pub resolution: IconResolution,
-}
-
 impl IconConversion {
     pub fn step_unincremented(&self, step_description: &str) {
         if let Some(progress_bar) = &self.progress_bar {
@@ -284,7 +287,7 @@ impl IconConversion {
 
     pub fn full_mask(
         &self,
-        options: &options::Options,
+        options: &Options,
         centering_dimensions: &Dimensions,
     ) -> Result<PathBuf, FolderifyError> {
         self.step_unincremented("Preparing icon mask");
@@ -300,7 +303,7 @@ impl IconConversion {
         args.extent(&Extent::no_offset(centering_dimensions));
         let output_path = self.output_path("0.0_FULL_MASK.png");
         args.push_path(&output_path);
-        run_convert(&args, None)?;
+        run_magick(&args, None)?;
         self.step("");
         Ok(output_path)
     }
@@ -321,7 +324,7 @@ impl IconConversion {
         });
         let output_path = self.output_path("1.0_SIZED_MASK.png");
         args.push_path(&output_path);
-        run_convert(&args, None)?;
+        run_magick(&args, None)?;
         Ok(output_path)
     }
 
@@ -337,7 +340,7 @@ impl IconConversion {
         let file_name = format!("{}.png", output_filename);
         let output_path = self.output_path(&file_name);
         args.push_path(&output_path);
-        run_convert(&args, None)?;
+        run_magick(&args, None)?;
         Ok(output_path)
     }
 
@@ -454,27 +457,42 @@ impl IconConversion {
         args.push_path(&top_bezel);
         args.composite(&CompositingOperation::dissolve);
         args.push_path(output_path);
-        run_convert(&args, Some(template_icon))?;
-
-        self.step("");
-
+        run_magick(&args, Some(template_icon))?;
         Ok(())
     }
 
+    pub fn badge_in_place(
+        &self,
+        icon_path: &Path,
+        badge: Badge,
+        resolution: &IconResolution,
+    ) -> Result<(), FolderifyError> {
+        self.step("Adding badge");
+
+        let badge_icon = get_badge_icon(badge, resolution);
+
+        let mut args = CommandArgs::new();
+        args.push_path(icon_path);
+        args.push("-");
+        args.composite(&CompositingOperation::dissolve);
+        args.push_path(icon_path);
+        run_magick(&args, Some(badge_icon))
+    }
+
     // TODO
     pub fn icon(
         &self,
         options: &Options,
         full_mask_path: &Path,
         output_path: &Path,
-        inputs: &IconInputs,
+        icon_inputs: &IconInputs,
     ) -> Result<(), FolderifyError> {
         // if options.verbose {
         //     println!("[Starting] {}", inputs.resolution);
         // }
 
-        let size = inputs.resolution.size();
-        let offset_y = inputs.resolution.offset_y();
+        let size = icon_inputs.resolution.size();
+        let offset_y = icon_inputs.resolution.offset_y();
 
         self.step_unincremented("Sizing mask");
         let sized_mask_path = self
@@ -492,11 +510,12 @@ impl IconConversion {
             .unwrap();
 
         // TODO
-        let template_icon = get_folder_icon(inputs.color_scheme, &inputs.resolution);
+        let template_icon = get_folder_icon(icon_inputs);
 
-        let fill_color = match inputs.color_scheme {
-            ColorScheme::Light => RGBColor::new(8, 134, 206),
-            ColorScheme::Dark => RGBColor::new(6, 111, 194),
+        let fill_color = match (icon_inputs.folder_style, icon_inputs.color_scheme) {
+            (FolderStyle::Tahoe, _) => RGBColor::new(74, 141, 172),
+            (_, ColorScheme::Light) => RGBColor::new(8, 134, 206),
+            (_, ColorScheme::Dark) => RGBColor::new(6, 111, 194),
         };
 
         let engraved = self.engrave(
@@ -516,14 +535,24 @@ impl IconConversion {
                 },
                 bottom_bezel: BezelInputs {
                     color: RGBColor::new(174, 225, 253),
-                    blur: inputs.resolution.bottom_bezel_blur_down(),
+                    blur: icon_inputs.resolution.bottom_bezel_blur_down(),
                     mask_operation: CompositingOperation::Dst_Out,
-                    opacity: inputs.resolution.bottom_bezel_alpha(),
+                    opacity: icon_inputs.resolution.bottom_bezel_alpha(),
                 },
             },
         );
+        if let Some(badge) = options.badge {
+            self.badge_in_place(output_path, badge, &icon_inputs.resolution)?;
+        };
+
+        self.step("");
+
         if options.verbose {
-            println!("[{}] {}", options.mask_path.display(), inputs.resolution);
+            println!(
+                "[{}] {}",
+                options.mask_path.display(),
+                icon_inputs.resolution
+            );
         }
         engraved
     }
@@ -564,6 +593,96 @@ impl IconConversion {
                 target_path.display(),
             );
         }
+
+        let assignment_fn = match options.set_icon_using {
+            SetIconUsing::Fileicon => Self::assign_icns_using_fileicon,
+            SetIconUsing::Osascript => Self::assign_icns_using_osascript,
+            SetIconUsing::Rez => Self::assign_icns_using_rez,
+        };
+        assignment_fn(self, options, icns_path, target_path)?;
+
+        self.step("");
+
+        Ok(())
+    }
+
+    pub fn assign_icns_using_osascript(
+        &self,
+        _options: &Options,
+        icns_path: &Path,
+        target_path: &Path,
+    ) -> Result<(), FolderifyError> {
+        self.step("Using `osascript` to assign the `.icns` file.");
+
+        let target_metadata = match metadata(target_path) {
+            Ok(target_metadata) => target_metadata,
+            Err(_) => {
+                eprintln!("Error: target path does not exist!");
+                exit(1);
+            }
+        };
+
+        // Adapted from:
+        // - https://github.com/mklement0/fileicon/blob/9c41a44fac462f66a1194e223aa26e4c3b9b5ae3/bin/fileicon#L268-L276
+        // - https://github.com/mklement0/fileicon/issues/32#issuecomment-1074124748
+        // - https://apple.stackexchange.com/a/161984
+        //
+        // In theory, we could try to call the Cocoa framework directly through
+        // bridging or linking. However, AppleScript is more likely to be
+        // portable across macOS versions.
+        let stdin = format!("use framework \"Cocoa\"
+
+            set sourcePath to \"{}\"
+            set destPath to \"{}\"
+            
+            set imageData to (current application's NSImage's alloc()'s initWithContentsOfFile:sourcePath)
+            (current application's NSWorkspace's sharedWorkspace()'s setIcon:imageData forFile:destPath options:2)",
+            escape_path_for_applescript(&icns_path.to_string_lossy()), escape_path_for_applescript(&target_path.to_string_lossy())
+        );
+
+        let args = CommandArgs::new();
+        run_command(OSASCRIPT_COMMAND, &args, Some(stdin.as_bytes()))?;
+
+        if target_metadata.is_dir() {
+            // TODO: check for network volume first, only then the appropriate path.
+            if metadata(target_path.join("Icon\r")).is_err()
+                && metadata(target_path.join(".VolumeIcon.icns")).is_err()
+            {
+                eprintln!("Icon was not successfully assigned to the target folder.");
+                exit(1);
+            }
+        } else {
+            // TODO: this is usually overwritten by the progress bars.
+            eprintln!(
+                "Target is not a folder. Please check manually if the icon was assigned correctly."
+            );
+        };
+
+        Ok(())
+    }
+
+    pub fn assign_icns_using_fileicon(
+        &self,
+        _options: &Options,
+        icns_path: &Path,
+        target_path: &Path,
+    ) -> Result<(), FolderifyError> {
+        self.step("Using `fileicon` to assign the `.icns` file.");
+        let mut args = CommandArgs::new();
+        args.push("set");
+        args.push_path(target_path);
+        args.push_path(icns_path);
+        run_command(FILEICON_COMMAND, &args, None)?;
+
+        Ok(())
+    }
+
+    pub fn assign_icns_using_rez(
+        &self,
+        _options: &Options,
+        icns_path: &Path,
+        target_path: &Path,
+    ) -> Result<(), FolderifyError> {
         let target_is_dir = metadata(target_path)
             .expect("Target does not exist!")
             .is_dir(); // TODO: Return `FolderifyError`
@@ -622,10 +741,13 @@ impl IconConversion {
             run_command(SETFILE_COMMAND, &args, None)?;
         } else {
             self.step("Skipping invisible file attribute for file target");
-        }
-
-        self.step("");
+        };
 
         Ok(())
     }
 }
+
+pub fn escape_path_for_applescript(path: &str) -> String {
+    // Newlines don't need to be escaped.
+    path.replace('\\', "\\\\").replace('\"', "\\\"")
+}
