@@ -1,20 +1,23 @@
 use std::thread::{self, JoinHandle};
 
 use command::{run_command, OPEN_COMMAND};
-use convert::CommandArgs;
-use icon_conversion::{IconInputs, IconResolution, WorkingDir};
+use icon_conversion::{IconResolution, WorkingDir};
 use indicatif::MultiProgress;
+use magick::CommandArgs;
+use shadow_rs::shadow;
 
-use crate::{output_paths::PotentialOutputPaths, primitives::Dimensions};
+use crate::{output_paths::PotentialOutputPaths, primitives::Dimensions, resources::IconInputs};
 
 mod command;
-mod convert;
 mod error;
-mod generic_folder_icon;
 mod icon_conversion;
+mod magick;
 mod options;
 mod output_paths;
 mod primitives;
+mod resources;
+
+shadow!(build);
 
 fn main() {
     let options = options::get_options();
@@ -22,8 +25,9 @@ fn main() {
     let potential_output_paths = PotentialOutputPaths::new(&options);
 
     println!(
-        "[{}] Using folder style: BigSur",
-        options.mask_path.display()
+        "[{}] Using folder style: {}",
+        options.mask_path.display(),
+        options.folder_style
     );
     println!(
         "[{}] Using color scheme: {}",
@@ -45,6 +49,7 @@ fn main() {
         icon_conversion::ProgressBarType::Input,
         "(Input)",
         multi_progress_bar.clone(),
+        &options,
     );
     let full_mask_path = input_icon_conversion
         .full_mask(
@@ -64,6 +69,7 @@ fn main() {
             icon_conversion::ProgressBarType::Conversion,
             &resolution.to_string(),
             multi_progress_bar.clone(),
+            &options,
         );
         let options = options.clone();
         let full_mask_path = full_mask_path.clone();
@@ -75,8 +81,10 @@ fn main() {
                     &full_mask_path,
                     &output_path,
                     &IconInputs {
+                        folder_style: options.folder_style,
                         color_scheme: options.color_scheme,
                         resolution,
+                        empty_folder: options.empty_folder,
                     },
                 )
                 .unwrap();
@@ -95,11 +103,15 @@ fn main() {
 
     // Deduplicate this `match` with the one that happens after handle joining.
     let output_progress_bar_type = match output_iconset_only {
-        Some(_) => icon_conversion::ProgressBarType::OutputWithoutIcns,
-        None => icon_conversion::ProgressBarType::OutputWithIcns,
+        Some(_) => icon_conversion::ProgressBarType::OutputIcns,
+        None => icon_conversion::ProgressBarType::OutputWithAssignment,
     };
-    let output_icon_conversion =
-        working_dir.icon_conversion(output_progress_bar_type, "(Output)", multi_progress_bar);
+    let output_icon_conversion = working_dir.icon_conversion(
+        output_progress_bar_type,
+        "(Output)",
+        multi_progress_bar,
+        &options,
+    );
     output_icon_conversion.step_unincremented("Waiting…");
 
     for handle in handles {
